*! version 1.0.1  26jul2004
program _coef_table
	version 8.2
	if "`e(cmd)'" == "" {
		error 301
	}
	syntax [,				///
		Level(integer `c(level)')	///
		PLus				///
		NOFirst				///
		First				///
		NEQ(integer -1)			///
		NODIPARM			///
		SEParator(integer 0)		///
		*				/// -eform/diparm- opts
	]

	if `: word count `first' `nofirst'' > 1 {
		di as err "options first and nofirst cannot be combined"
		exit 198
	}

	// parse `options' for -eform()- and friends
	_get_eformopts , eformopts(`options') allowed(__all__) soptions
	local eform	`"`s(str)'"'
	local udiparm	`"`s(options)'"'
	// `udiparm' should only contain -diparm()- options
	_get_diparmopts, diparmopts(`udiparm') level(`level')

	// ignore -diparm()- options; but checked for valid syntax anyway
	if "`nodiparm'" != "" {
		local udiparm
	}

	// get equation names for auxiliary parameters or extra equations
	local coleq : coleq e(b), quote
	local coleq : list clean coleq
	local coleq : list uniq coleq

	Chk4Scalars k_eq k_aux k_extra
	// check for total number of equations
	local k_eq 0
	Chk4PosInt k_eq
	if `k_eq' == 0 {
		local k_eq : word count `coleq'
	}
	else if `k_eq' > `:word count `coleq'' {
		di as err ///
"estimation command error: e(k_eq) is larger than the number of equations"
		exit 322
	}
	// check for auxiliary parameters
	local k_aux 0
	Chk4PosInt k_aux
	// check for extra equations
	local k_extra 0
	Chk4PosInt k_extra

	// check that the e(k_*) scalars add up correctly
	if `k_aux'`k_extra' {
		if `:word count `coleq'' != `k_eq' {
			di as err ///
"estimation command error: e(k_eq) does not equal the number of equations"
			exit 322
		}
	}

	// figure out how many equations -ereturn- is going to display
	if "`first'" != "" {
		local neq 1
	}
	if `neq' > 0 {
		// user only wants the `neq' equations
		local k_aux 0
		local k_extra 0
	}
	else if `neq' < 1 {
		// automatically determine neq
		local neq = `k_eq'-`k_aux'-`k_extra'
		if `neq' < 0 {
			if `k_aux' & `k_extra' {
				di as err ///
"estimation command error: e(k_eq) is less than e(k_aux) + e(k_extra)"
				exit 322
			}
			local name = cond(`k_aux',"k_aux","k_extra")
			di as err ///
"estimation command error: e(k_eq) is less than e(`name')"
			exit 322
		}
		local SvySum mean total ratio proportion
		local cmd = cond("`e(cmd2)'"!="","`e(cmd2)'","`e(cmd)'")
		if !`:list cmd in SvySum' & `neq' == 1 ///
		 & "`nofirst'" == "" & `k_extra' == 0 {
			local first first
		}
	}

	local k_eform 0
	if inlist("mlogit","`e(cmd)'","`e(cmd2)'") {
		local k_eform = `k_eq'-`k_aux'-`k_extra'
	}
	else if `neq' {
		local k_eform 1
		Chk4NNInt k_eform
		if `k_eform' > `neq' {
			di as err ///
"estimation command error: e(k_eform) is too large"
			exit 322
		}
	}

	local copy `"`coleq'"'
	forval j = 1/`k_eform' {
		gettoken eq copy : copy, quotes
		local efcoleq `"`efcoleq' `eq'"'
	}
	forval j = `=`k_eform'+1'/`neq' {
		gettoken eq copy : copy, quotes
		local noefcoleq `"`noefcoleq' `eq'"'
	}

	// auxiliary parameters, and their transforms
	if `k_aux' {
		local i1 = `neq' + 1
		local i2 = `neq' + `k_aux'
		local isep 0
		local COLON
		forval i = `i1'/`i2' {
			if `isep' & mod(`isep',`separator') == 0 {
				local COLON ":"
				local diparm `"`diparm' `COLON' __sep__"'
			}
			local eq : word `i' of `coleq'
			if `"`e(diparm_opt`i')'"' != "" {
				local eq `"`eq', `e(diparm_opt`i')'"'
			}
			local diparm `diparm' `COLON' `eq'
			local ++isep
			local COLON ":"
		}
		if `"`diparm'"' != "" {
			local diparm diparm(`diparm')
		}
	}
	if "`nodiparm'" == "" &	`"`e(diparm)'`e(diparm1)'`udiparm'"' != "" {
		GetEDiparm ediparm
		if `"`ediparm'"' != "" {
			if `"`diparm'"' != "" {
				local diparm `diparm' diparm(__sep__)
			}
			local diparm `diparm' `ediparm'
		}
		if "`udiparm'" != "" {
			if `"`diparm'"' != "" {
				local diparm `diparm' diparm(__sep__)
			}
			local diparm `diparm' `udiparm'
		}
	}
	// display extra equations
	if `k_extra' {
		local i1 = `neq' + `k_aux' + 1
		forval i = `i1'/`k_eq' {
			local eq : word `i' of `coleq'
			local extra `"`extra' "`eq'""'
		}
	}

	// display any constraints
	tempname cns error
	capture mat `cns' = get(Cns)
	if !c(rc) {
		matrix dispCns
	}

	local df_r .
	Chk4Int df_r

	local emats : e(matrices)
	if `:list posof "error" in emats' {
		matrix `error' = e(error)
		local erropt error(`error')
	}

	tempname z
	if missing(`df_r') {
		scalar `z' = invnorm((100+`level')/200)
	}
	else	scalar `z' = invttail(`df_r',(100-`level')/200)

	tempname Tab
	.`Tab' = ._tab.new, col(7) lmargin(0) ignore(.b)
	// column        1      2     3     4     5     6     7
	.`Tab'.width	13    |12    11     9     8    12    12
	.`Tab'.titlefmt  .      .  %11s   %7s     .  %24s     .
	.`Tab'.pad	 .      2     1     0     2     3     3
	.`Tab'.numfmt    .  %9.0g %9.0g %8.2f %5.3f %9.0g %9.0g
	.`Tab'.strcolor  . result    .     .     .     .     .

	if "`e(clustvar)'" != "" {
		di as txt ///
"{ralign 78:(standard errors adjusted for clustering on `e(clustvar)')}"
	}
	ColumnTitles `Tab' `level' `df_r' `"`eform'"' `neq'

	if `"`efcoleq'"' != "" {
		DisplayEqns `efcoleq' :		///
			`Tab',	l(`level')	///
				df(`df_r')	///
				z(`z')		///
				ef(`eform')	///
				`first'		///
				`erropt'	///
				// blank
	}
	if `"`noefcoleq'"' != "" {
		DisplayEqns `noefcoleq' : ///
		`Tab', l(`level') df(`df_r') z(`z') `erropt'
	}
	if `"`diparm'"' != "" {
		.`Tab'.sep
		_get_diparmopts, level(`level')	diparmopts(`diparm') execute
	}
	if `"`extra'"' != "" {
		DisplayEqns `extra' : `Tab', l(`level') df(`df_r') z(`z')
	}
	if "`plus'" == "" {
		.`Tab'.sep, bottom
	}
	else	.`Tab'.sep

	if e(rc) == 504 & "`e(opt)'" == "ml" {
		di as err ///
"{p 0 0 2}variance matrix missing because `e(user)' failed" ///
"to compute scores or computed scores with missing values{p_end}"
		exit 504
	}
	if (!missing(e(rc)) & e(rc) != 0) error e(rc)
end

program ColumnTitles
	args Tab level df eform neq

	local SvySum mean total ratio proportion
	local cmd = cond("`e(cmd2)'"!="","`e(cmd2)'","`e(cmd)'")

	if `:list cmd in SvySum' {
		if "`e(by)'" != "" {
			if `:word count `e(by)'' != 1 {
				if "`e(cmd)'" == "svy" {
					local depvar "Subpop."
				}
				else	local depvar "By"
			}
			else if "`e(cmd)'" != "svy" {
				local depvar = abbrev("`e(by)'",12)
			}
		}
		local coef = proper("`cmd'")
	}
	else {
		local coleq : coleq e(b)
		local dv `"`e(depvar)'"'
		if `:word count `dv'' == 1 ///
		 & (`neq' <= 1 | !`:list dv in coleq') {
			local depvar = abbrev("`dv'",12)
		}
		if "`eform'" != "" {
			local coef "`eform'"
		}
		else	local coef "Coef."
	}
	if missing(`df') {
		local stat z
	}
	else	local stat t
	.`Tab'.sep, top
	if `"`e(vcetype)'"' != "" {
		local vcetype `"`e(vcetype)'"'
		local vcewd : length local vcetype
		if `vcewd' <= 12 {
			local vcewd = `vcewd' + ceil((12-`vcewd')/2)
		}
		if "`e(vcetype)'" == "Bootstrap" {
			local obs "Observed"
		}
		// column        1       2                3   4  5  6  7
		.`Tab'.titlefmt  .       .        %`vcewd's   .  .  .  .
		.`Tab'.titles   "" "`obs'" `" `e(vcetype)'"' "" "" "" ""
		.`Tab'.titlefmt  .       .             %11s   .  .  .  .
	}
	.`Tab'.titles	"`depvar'"			/// 1
			"`coef'"			/// 2
			"Std. Err."			/// 3
			"`stat'"			/// 4
			"P>|`stat'|"			/// 5
			"[`level'% Conf. Interval]" ""	//  6 7
end

program DisplayEqns
	_on_colon_parse `0'
	local eqlist `"`s(before)'"'
	if (`: word count `eqlist'' == 0) exit

	local 0 `s(after)'
	syntax name(name=Tab) [,	///
		First			///
		EForm(passthru)		///
		ERRormat(name)		///
		*			///
	]

	local coleq : coleq e(b), quote
	local coleq : list clean coleq
	local colna : colna e(b)
	local neq   : word count `coleq'

	// index of first element
	local ucoleq : list uniq coleq
	local start : list posof "`:word 1 of `eqlist''" in coleq

	// look for time series operators in the equations to be displayed
	local i `start'
	while `i' <= `neq' {
		local eq : word `i' of `coleq'
		local qeq `""`eq'""'
		// exit if the current equation is not in the list
		local break = (!`:list eq in eqlist' & !`:list qeq in eqlist')
		if (`break') continue, break
		local na : word `i' of `colna'
		capture tsunab na : `na'
		if !c(rc) {
			if "`s(tsops)'" == "true" {
				local tsops tsops
				continue, break
			}
		}
		local ++i
	}

	// loop through the coefficients, displaying those that belong to the
	// specified equations

	local i `start'
	while `i' <= `neq' {
		local eq : word `i' of `coleq'
		local qeq `""`eq'""'
		// exit if the current equation is not in the list
		local break = (!`:list eq in eqlist' & !`:list qeq in eqlist')
		if (`break') continue, break
		local na : word `i' of `colna'
		if `"`eq'"' != `"`oldeq'"' {
			if "`offset'" != "" {
				// display offset of previous equation
				Offset `Tab' `"`offset'"' `"`offtyp'"'
				local offset
				local offtyp
			}
			.`Tab'.sep
			if "`first'" == "" & `"`eq'"' != "_" {
				local abeq = abbrev(`"`eq'"',12)
				// columns           1   2  3  4  5  6  7
				.`Tab'.strcolor result   .  .  .  .  .  .
				.`Tab'.strfmt    %-12s   .  .  .  .  .  .
				.`Tab'.row   `"`abeq'"' "" "" "" "" "" ""
				.`Tab'.strcolor   text   .  .  .  .  .  .
				.`Tab'.strfmt     %12s   .  .  .  .  .  .
			}
			local oldeq `"`eq'"'

			// get offset of current equation
			local ieq : list posof `"`eq'"' in ucoleq
			_get_offopt `e(offset)'`e(offset`ieq')'
			local offset `s(offvar)'
			if "`s(offopt)'" == "offset(`offset')" {
				local offtyp "(offset)"
			}
			else if "`s(offopt)'" != "" {
				local offtyp "(exposure)"
			}
		}
		if `"`eform'"' == "" | "`na'" != "_cons" {
			if "`errormat'" != "" {
				local erropt error(`=`errormat'[1,`i']')
			}
			Row `Tab' `"`qeq'"' "`na'", ///
				`options' `eform' `tsops' `erropt'
		}
		local ++i
	}
	if "`offset'" != "" {
		// display offset of last displayed equation
		Offset `Tab' `"`offset'"' `"`offtyp'"'
		local offset
		local offtyp
	}
end

program Offset
	args Tab offset offtyp
	local offset = abbrev("`offset'", 12)
	.`Tab'.strcolor           .         text   .  .  .  .  .
	.`Tab'.row	`"`offset'"' `"`offtyp'"' "" "" "" "" ""
	.`Tab'.strcolor           .       result   .  .  .  .  .
end

program Row
	syntax anything [,			///
		Level(integer `c(level)')	///
		EForm(passthru)			///
		ERRor(integer 0)		///
		df(real 0)			///
		z(name)				///
		noTEST				///
		noCI				///
		TSops				///
	]
	local 0 `anything'
	args Tab eq na
	local eq : list clean eq
	tempname b se t p ll ul
	if `"`eq'"' != "_" {
		scalar `b'  = [`eq']_b[`na']
		scalar `se' = [`eq']_se[`na']
	}
	else {
		scalar `b'  = _b[`na']
		scalar `se' = _se[`na']
	}
	if `se' == 0 {
		scalar `se' = .
	}
	if "`notest'" == "" {
		scalar `t' = `b'/`se'
		if missing(`df') {
			scalar `p' = 2*norm(-abs(`t'))
		}
		else	scalar `p' = 2*ttail(`df',abs(`t'))
	}
	else {
		scalar `t' = .b
		scalar `p' = .b
	}
	if "`noci'" == "" {
		scalar `ll' = `b' - `se'*`z'
		scalar `ul' = `b' + `se'*`z'
	}
	else {
		scalar `ll' = .b
		scalar `ul' = .b
	}
	if "`eform'" != "" {
		scalar `b'  = exp(`b')
		scalar `se' = `b'*`se'
		scalar `ll' = exp(`ll')
		scalar `ul' = exp(`ul')
	}

	if "`tsops'" != "" {
		// "." is assumed the separator for time series operators
		// i.e. <op>.<name>

		.`Tab'.strfmt    %-12s  .  .  .  .  .  .
		capture tsunab na : `na'
		if !c(rc) {
			gettoken op na : na, parse(".")
			if `:length local op' == 1 {
				local op `op'1
			}
			local na = abbrev(substr("`na'",2,.),12)
			// columns           1  2  3  4  5  6  7
			.`Tab'.row    `"`na'"' "" "" "" "" "" ""
			.`Tab'.strfmt     %12s  .  .  .  .  .  .
			local na `op'
		}
	}
	else	.`Tab'.strfmt     %12s  .  .  .  .  .  .
	local na = abbrev("`na'",12)

	local error1 "  (no observations)"
	local error2 "  (stratum with 1 PSU detected)"
	local error3 "  (sum of weights equals zero)"
	local error4 "  (denominator estimate equals zero)"
	local error5 "(dropped)"

	if `error' {
		.`Tab'.row "`na'" "`error`error''" "" "" "" "" ""
	}
	else if `b'== 0 & `se' == . {
		.`Tab'.row "`na'" "`error5'" "" "" "" "" ""
	}
	else {
		.`Tab'.row "`na'" `b' `se' `t' `p' `ll' `ul'
	}
end

program GetEDiparm
	args c_diparm
	if `"`e(diparm)'`e(diparm1)'"' != "" {
		if `"`e(diparm)'"' != "" {
			local i 0
			local ediparmi `"`e(diparm)'"'
		}
		else if `"`e(diparm1)'"' != "" {
			local i 1
			local ediparmi `"`e(diparm1)'"'
		}
	}
	while `"`ediparmi'"' != "" {
		local diparm `diparm' diparm(`ediparmi')
		local ++i
		local ediparmi `"`e(diparm`i')'"'
	}
	c_local `c_diparm' `diparm'
end

program Chk4Scalars
	syntax namelist
	local nonscalars `: r(macros)' `: r(matrices)'
	local badmac : list nonscalars & namelist
	if "`badmac'" != "" {
		gettoken name : badmac
		di as err "type mismatch: e(`name') is not a scalar"
		exit 109
	}
end

program Chk4PosInt
	args ename
	if `"`e(`ename')'"' != "" {
		capture confirm integer number `e(`ename')'
		if !c(rc) {
			if `e(`ename')' > 0 {
				c_local `ename' `e(`ename')'
			}
		}
	}
end

program Chk4NNInt
	args ename
	if `"`e(`ename')'"' != "" {
		capture confirm integer number `e(`ename')'
		if !c(rc) {
			if `e(`ename')' >= 0 {
				c_local `ename' `e(`ename')'
			}
		}
	}
end

program Chk4Int
	args ename
	if `"`e(`ename')'"' != "" {
		capture confirm integer number `e(`ename')'
		if !c(rc) {
			c_local `ename' `e(`ename')'
		}
	}
end

exit

NOTES:

-_coef_table- looks at the following scalars to determine which equations are
put in the table of results, and how they are put there:

	e(k_eq)		-- total number of equations, missing value implies 1
	e(k_aux)	-- number of ancillary parameters (each an equation)
	e(k_extra)	-- extra equations
	e(k_eform)	-- the first `e(k_eform)' equations' coefficients will
			   be exponentiated when an 'eform' type option is
			   specified; default is 1

