*! version 1.1.12  17oct2003
program lowess, sortpreserve
	version 8.0
	if _caller() < 8 {
		lowess_7 `0'
		exit
	}

	syntax varlist(min=2 max=2 numeric)	///
		[if] [in] [,			///
		noGraph				///
		GENerate(string)		///
		Adjust				///
		LOgit				///
		Sort				/// [overridden]
		BWidth(real 0)			/// _LOWESS opts
		Mean				///
		noWeight			///
		*				/// graph opts
	]

	_get_gropts , graphopts(`options') grbyable		///
		getbyallowed(TItle SUBTItle note legend)	///
		getallowed(rlopts plot)
	local by `s(varlist)'
	local byopts `"`s(byopts)'"'
	local bytitle `"`s(by_title)'"'
	local bysubtitle `"`s(by_subtitle)'"'
	local bynote `"`s(by_note)'"'
	local bylegend `"`s(by_legend)'"'
	local options `"`s(graphopts)'"'
	local rlopts `"`s(rlopts)'"'
	local plot `"`s(plot)'"'
	_check4gropts rlopts, opt(`rlopts')

	if "`generate'" != "" {
		confirm new var `generate'
	}

	local lopts "`mean' `weight'"
	local weight 
	marksample touse

	local sort "sort"	/* Force -sort- */

	tokenize `varlist'
	local y `1'
	local x `2'

	tempvar grp
	if "`by'" != "" {
		sort `touse' `by', stable
		qui by `touse' `by': gen long `grp'=1 if _n==1 & `touse'
		qui replace `grp'=sum(`grp') if `touse'
		local n = `grp'[_N]
		if `n' < 2 {
			di in red "by() variable takes on only one value"
			exit 198
		}
	}
	else {
		local n=1
		qui gen `grp'=1
	}
	local bw = `bwidth'
	if `bw' <= 0 | `bw' >= 1 {
		local bw .8
	}

	tempvar subuse ys sm
	qui gen byte `subuse' = .
	qui gen double `ys' = .
	qui gen double `sm' = .

	/* Compute smooth for each group */
	forval i = 1/`n' {
		qui replace `subuse'=`touse' & `grp'==`i'
		summ `x' if `subuse', mean
		if r(N) < 3 {
			noisily error 2001
		}
		if r(max)-r(min) < 1e-30 {
			di in red "Range of `x' is too small"
			exit 499
		}

		noi _LOWESS `y' `x' if `subuse',	///
			lowess(`ys')			///
			bwidth(`bw')			///
			`lopts'

		qui ADJ_LOGIT `y' `ys' if `subuse',	///
			`adjust' `logit'

		qui replace `sm' = `ys' if `subuse'
	}

	/* Graph (if required) */
	if "`graph'" == "" { 
		if `"`bytitle'"' == "" {
			if "`mean'" == "mean" {
				local title title("Running mean smoother")
			}
			else if "`weight'" == "noweight" {
				local title title("Running line smoother")
			}
			else	local title title("Lowess smoother")
		}
		if `"`bysubtitle'"' == "" {
			if `"`logit'"' != "" {
				local subttl1 ///
				subtitle(`"Logit transformed smooth"')
				local yttl `"logit of `yttl'"'
				local suffix suffix
			}
			if `"`adjust'"' != "" {
				local subttl2 ///
				subtitle(`"Mean adjusted smooth"', `suffix') 
			}
		}
		local yttl : var label `y'
		if `"`yttl'"' == "" {
			local yttl `y'
		}
		local xttl : var label `x'
		if `"`xttl'"' == "" {
			local xttl `x'
		}
		if `"`bynote'"' == "" {
			local note note(`"bandwidth = `bw'"')
		}
		if "`logit'" == "" {
			local symbol .
		}
		else {
			local symbol none
		}
		if `"`plot'"' == "" & `"`bylegend'"' == "" {
			local legend legend(nodraw)
		}
		label var `sm' "lowess: `yttl'"
		local titles				///
			`title'				///
			`subttl1'			///
			`subttl2'			///
			`note'				///
			`legend'			///
			// blank
		if `"`by'"' != "" {
			local byopt by(			///
				`by',			///
				`bytitle'		///
				`bysubtitle'		///
				`bynote'		///
				`bylegend'		///
				`titles'		///
			)
			local titles
		}
		graph twoway				///
		(scatter `y' `x' if `touse',		///
			sort				///
			msymbol(`symbol')		///
			`titles'			///
			ytitle(`"`yttl'"')		///
			xtitle(`"`xttl'"')		///
			`byopt'				///
			`options'			///
		)					///
		(line `sm' `x' if `touse',		///
			sort				///
			pstyle(p1)			///
			`rlopts'			///
		)					///
		|| `plot'				///
		// blank
	}

	if "`generate'" != "" {
		rename `sm' `generate'
	}
end

program ADJ_LOGIT	/* ADJ_LOGIT y ys if ... , ... */
	syntax varlist(min=2 max=2 numeric) if [, adjust logit ]

	marksample touse
	tokenize `varlist'
	local y `1'		/* y values */
	local ys `2'		/* smoothed values */

	/* Adjust smooth so that mean of smoothed values equals mean of y
	 * values.  */

	if "`adjust'" == "adjust" {
		tempname mean
		summ `ys' , mean
		scalar `mean' = r(mean)
		summ `y' , mean
		replace `ys' = `ys'* r(mean)/`mean' if `touse'
	}

	/* Perform logit transform of smoothed values */

	if "`logit'" == "logit" {
		tempname adj small
		qui count if `touse'
		scalar `adj' = 1/r(N)
		scalar `small' = 0.0001
		replace `ys' = `adj' if `ys' < `small' & `touse'
		replace `ys' = 1-`adj' if `ys' > (1-`small') & `touse'
		replace `ys' = ln(`ys'/(1-`ys')) if `touse'
	}
end

exit

