*! version 1.0.1  21aug2003
program ml_technique, sclass
	version 8.0
	// parse the -technique()- and -vce()- options of -ml-
	syntax [,				///
		TECHnique(string)		///
		VCE(string)			///
	]

	sreturn clear

	// saved results:
	//   s(vce)
	get_vce , `vce'

	// saved results:
	//   s(techlist)
	//   s(numlist)
	//   s(technique)
	get_technique `technique'
end

program get_vce, sclass
	syntax [,				///
		Hessian				/// VCE types
		OIM				///
		Opg				///
		NATIVE				///
	]
	// OIM is a synonym for Hessian
	if "`oim'" != "" {
		local hessian hessian
	}
	local vce `hessian' `opg' `native'
	if `: word count `vce'' > 1 {
		local vce : list retok vce
		di as err "options `vce' may not be combined in vce()"
		exit 198
	}
	if "$ML_vce" == "robust" & "`vce'" != "" & "`vce'" != "hessian" {
		di as err "options robust and vce(`vce') may not be combined"
		exit 198
	}
	sreturn local vce `vce'
end

// Syntax:
//   get_technique [ tname [#] ]...
//
// NOTE: The default value of # is 1.
// NOTE: The order of the arguments is preserved on purpose.
//
// Saved results:
//
//   s(techlist)   list of <tname>s in order given
//   s(numlist)    list of #s in order given, with default #s
//   s(technique)  retokenized arguments, with default #s
//
program get_technique, sclass
	local default_num 5
	gettoken tname rest : 0, parse(" =")
	while "`tname'" != "" {
		local tname = lower(`"`tname'"')
		capture confirm name `tname'
		if _rc {
			local rc = _rc
			di as err ///
			"`tname' is an invalid name in technique() option"
			exit `rc'
		}
		get_tech tech , `tname'
		gettoken num : rest, parse(" =")
		if "`num'" == "=" {
			// remove optional "=", UNDOCUMENTED
			gettoken num rest : rest, parse(" =")
			gettoken num : rest, parse(" =")
		}
		capture confirm integer number `num'
		if !_rc	gettoken num rest : rest, parse(" =")
		else	local num `default_num'
		if `num' < 0 {
			di as err ///
"negative integers are not allowed in technique() option"
			exit 125
		}
		else if `num' > 0 {
			local techlist `techlist' `tech'
			local numlist `numlist' `num'
			local technique `technique' `tech' `num'
		}
		gettoken tname rest : rest, parse(" =")
	}
	if "`techlist'" == "" {
		local techlist nr
		local numlist `default_num'	// will be ignored
	}

	sreturn local techlist `techlist'
	sreturn local numlist `numlist'
	sreturn local technique `technique'
end

program get_tech
	syntax name(id="macro name") [,		///
		BHHH				/// current valid techniques
		BHHHQ				///
		BFGS				///
		DFP				///
		NR				///
		*				/// invalid technique
	]
	if "`options'" != "" {
		di as err ///
		"`options' is not a valid argument of technique() option"
		exit 198
	}
	c_local `namelist' `bhhh' `bhhhq' `bfgs' `dfp' `nr'
end
