*! version 1.1.4  13apr2004
program numlabel
	version 8, born(30apr2004)

	syntax [anything(id=labellist name=names)] ///
		[, Add Remove Mask(str) Detail force ]

	if "`add'" != "" & "`remove'" != "" {
		dis as err "options add and remove must not be both specified"
		exit 198
	}
	if "`add'" == "" & "`remove'" == "" {
		dis as err "option add or remove should be specified"
		exit 198
	}
	if `"`mask'"' != "" {
		loc tmp: subinstr local mask "#" "", all count(local nch)
		if `nch' != 1 {
			dis as err `"mask `mask' should contain "' ///
			           `"1 substitution char #"'
			exit 198
		}
	}
	else 	local mask "#. "

// parsing is ready

	preserve
	uselabel `names', clear
	if _N==0 {
		exit
	}

	// check keyword for Stata/SE
	local strlen = cond("$S_StataSE" == "", 80, 244)

	gen str`strlen' prefix = subinstr(`"`mask'"', "#", string(value), 1)
	qui compress
	gen numlab = (prefix == substr(label,1,length(prefix)))

	if "`force'" == "" {
		capt bys lname (numlab) : assert numlab[1] == numlab[_N]
		if _rc {
			if "`detail'" != "" {
				dis _n as txt "The following value labels " ///
				              "are only partially numlabel-ed"
				qui bys lname (numlab): ///
				    drop if numlab[1] == numlab[_N]
				bys lname (value) : ///
				    list value label, noobs sep(0)
			}
			else {
				dis as txt ///
				"there are value labels that are only partially numlabel-ed" _n ///
				"specify option detail for a list of these value labels"
			}
			dis as txt "specify option force to modify value " ///
			           "labels anyway"
			exit 180
		}
	}

	if "`add'" != "" {
		gen newlen = cond(numlab, length(label), ///
		                  length(prefix)+length(label))
		qui count if newlen > `strlen'
		if r(N) > 0 {
			dis as err "impossible to prefix a value " ///
			           "to value labels (>`strlen' chars)"
			if "`detail'" != "" {
				qui keep if newlen > `strlen'
				bys lname (value) : list value label, noobs
			}
			exit 180
		}
		gen str`strlen' modlabel = cond(numlab, label, prefix+label)
	}

	if "`remove'" != "" {
		gen str`strlen' modlabel = cond(numlab, ///
		          substr(label, length(prefix)+1,.), label)
	}

// re-apply the modified labels

	qui keep if label != modlabel
	if _N == 0 {
		dis as txt "(no value label to be modified)"
		exit 0
	}
	qui compress modlabel

	forvalues i = 1 / `=_N' {
		local lname = lname[`i']
		local value = value[`i']
		local mlabel = modlabel[`i']
		label define `lname' `value' `"`macval(mlabel)'"' , modify
	}

	// create file with label definitions
	tempfile f
	qui label save using `"`f'"'

	// bring back original data, and run definition file
	restore
	run `"`f'"'
end
exit

HISTORY

1.1.2  added support for Stata/SE

1.1.1  improved syntax parsing via syntax:anything
       modified wording in some err msg
