*! version 1.0.2  30sep2004
program define xtgee_p
	version 6, missing

		/* Step 1:
			place command-unique options in local myopts
			Note that standard options are
			LR:
				Index XB Cooksd Hat 
				REsiduals RSTAndard RSTUdent
				STDF STDP STDR noOFFset
			SE:
				Index XB STDP noOFFset
		*/
	local myopts "MU Rate"


		/* Step 2:
			call _propts, exit if done, 
			else collect what was returned.
		*/
	_pred_se "`myopts'" `0'
	if `s(done)' { exit }
	local vtyp  `s(typ)'
	local varn `s(varn)'
	local 0 `"`s(rest)'"'


		/* Step 3:
			Parse your syntax.
		*/
	syntax [if] [in] [, `myopts' noOFFset]


		/* Step 4:
			Concatenate switch options together
		*/
	local type "`mu'`rate'"


		/* Step 5:
			quickly process default case if you can 
			Do not forget -nooffset- option.
		*/


		/* Step 6:
			mark sample (this is not e(sample)).
		*/
	marksample touse


		/* Step 7:
			handle options that take argument one at a time.
			Comment if restricted to e(sample).
			Be careful in coding that number of missing values
			created is shown.
			Do all intermediate calculations in double.
		*/


		/* Step 8:
			handle switch options that can be used in-sample or 
			out-of-sample one at a time.
			Be careful in coding that number of missing values
			created is shown.
			Do all intermediate calculations in double.
		*/

	if "`type'"=="" | "`type'"=="mu" { 
		Title `offset'
		local ttl `"`s(title)'"'
		if "`type'"=="" {
			di in gr `"(option mu assumed; `ttl)')"'
		}
		Calc "`vtyp'" `varn' `touse' `offset'
		label var `varn' `"`ttl'"'
		exit
	}

	if "`type'"=="rate" {
		Title nooffset
		local ttl `"`s(title)'"'
		Calc "`vtyp'" `varn' `touse' nooffset
		label var `varn' `"`ttl'"'
		exit
	}
		

		/* Step 9:
			handle switch options that can be used in-sample only.
			Same comments as for step 8.
		*/
	* qui replace `touse'=0 if !e(sample)


			/* Step 10.
				Issue r(198), syntax error.
				The user specified more than one option
			*/
	error 198
end

program define Title, sclass
	args offset
	sret clear

	if "`e(denom)'" != "" & "`e(denom)'"!="1" { 
		if "`offset'"=="" {
			local denom `e(denom)'
			local std `"`denom'*"'
		}
	}

	if "`e(link)'"== "identity" {
		sret local title "`std'xb"
	}
	else if "`e(link)'"== "log" {
		sret local title "`std'exp(xb)"
	}
	else if "`e(link)'"== "logit" {
		sret local title "`std'exp(xb)/(1+exp(xb))"
	}
	else if "`e(link)'"== "cloglog" {
		sret local title "`std'(1-exp(-exp(xb)))"
	}
	else if "`e(link)'"== "reciprocal" {
		if "`denom'" == "" {
			sret local title "1/xb"
		}
		else	sret local title "`denom'/xb"
	}
	else if "`e(link)'"== "probit" {
		sret local title "`std'normprob(xb)"
	}
	else if substr("`e(link)'",1,5)== "power" {
		sret local title "`std'xb^(1/`e(power)')"
	}
	else if substr("`e(link)'",1,5)== "odds p" {
		sret local title "`std'inv_odds_power(xb,`e(power)')"
	}
	else if "`e(link)'"== "nbinomial" {
		sret local title "`std'k*exp(xb)/(1-exp(xb))"
	}
	else	error 301
end

program define Calc
	args vtyp varn touse offset

	tempvar xb
	qui _predict double `xb' if `touse', xb `offset'

	local std 1
	if "`e(denom)'" != "" & "`e(denom)'"!="1" { 
		if "`offset'"=="" {
			local std `"`e(denom)'"'
		}
	}

	if "`e(link)'"== "identity" {
		gen `vtyp' `varn' = `std'*`xb' if `touse'
	}
	else if "`e(link)'"== "log" {
		gen `vtyp' `varn' = `std'*exp(`xb') if `touse'
	}
	else if "`e(link)'"== "logit" {
		gen `vtyp' `varn' = `std'*exp(`xb')/(1+exp(`xb')) if `touse'
	}
	else if "`e(link)'"== "cloglog" {
		gen `vtyp' `varn' = `std'*(1-exp(-exp(`xb'))) if `touse'
	}
	else if "`e(link)'"== "reciprocal" {
		gen `vtyp' `varn' = `std'/`xb' if `touse'
	}
	else if "`e(link)'"== "probit" {
		gen `vtyp' `varn' = `std'*normprob(`xb') if `touse'
	}
	else if substr("`e(link)'",1,5)== "power" {
		gen `vtyp' `varn' = `std'*`xb'^(1/`e(power)') if `touse'
	}
	else if substr("`e(link)'",1,5)== "odds p" {
		qui replace `xb' = (`e(power)'*`xb'+1)^(1/`e(power)') /*
			*/ if `touse'
		gen `vtyp' `varn' = `std'*`xb'/(1+`xb') if `touse'
	}
	else if "`e(link)'"== "nbinomial" {
		gen `vtyp' `varn' = /*
			*/ `std'*`e(nbalpha)'*exp(`xb')/(1-exp(`xb')) /*
			*/ if `touse'
	}
	else	error 301
end
